var underscore = require('underscore'),
    couchdb = require('couchdb'),
    utils = require('./utils');

Storage = function(settings) {
    this._settings = settings;
    this._confs = {};
    this._stores = {};
    var self = this;
    var configure = function() {
        var file = self._settings.database_auth_file;
        var passwords = jsonFromFile(file);
        if (!passwords) {
            throw new Error('Could not load database auth file: ' + file);
        }
        var databases = self._settings.databases;
        var rawConfs = _.isArray(databases) ? databases : [databases];
        _.each(rawConfs, function(rawConf) {
            var key = rawConf.key;
            var password = passwords[rawConf.username].password;
            var defaults = {
                port: 5984,
                host: 'localhost'
            };
            rawConf = _.extend(defaults, rawConf);
            rawConf.password = password;
            self._confs[key] = rawConf;
        });
    };
    var setupStores = function() {
        _.each(self._confs, function(conf, name) {
            self.getDatabase(name);
        });
    };
    configure();
    setupStores();
}

Storage.prototype.getDatabaseConf = function(name) {
    return this._confs[name];
};

Storage.prototype.getDatabaseClient = function(conf) {
    return couchdb.createClient(
        conf.port,
        conf.host, 
        conf.username, 
        conf.password
    );
};

Storage.prototype.getDatabase = function(name) {
    if (this._stores[name]) {
        return this._stores[name];
    }
    var conf = this.getDatabaseConf(name);
    var client = this.getDatabaseClient(conf);
    var db = client.db(conf.name);
    this._stores[name] = db;
    return db;
};


Storage.prototype.getDatabases = function() {
    return this._stores;
};

exports.Storage = Storage;
